#!/usr/bin/env python3

"""Configuration and context information for dragonglass"""

from argparse import Namespace
from pathlib import Path
from typing import Any

import tomllib

from .tree import SourceIndex, SourceNode


"""The default configuration file name."""
DEFAULT_CONFIG_FILE = ".dragonglass"
"""The default template directory name."""
DEFAULT_TEMPLATE_DIRECTORY = ".dragonglass.tmpl"
"""The default template name."""
DEFAULT_TEMPLATE_NAME = "default.html"
""" The default stylesheet name."""
DEFAULT_STYLESHEET_NAME = "dragonglass.css"


class Context:
    """
    The configuration and context information for dragonglass.

    Attributes:
        source_dir (Path): The source directory (Obsidian vault) for the operation.
        template_dir (Path): The directory in which templates may be found.
        config (dict[str, Any]): The configuration read in from the default configuration file.
        src_index (SourceIndex): The index of all nodes, used to look up targets for internal links.
        current_node (SourceNode): The current node being processed.
    """
    def __init__(self) -> None:
        """Initialize the Context."""
        self.source_dir: Path | None = None
        self.template_dir: Path | None = None
        self.config: dict[str, Any] = {}
        self.src_index: SourceIndex | None = None
        self.current_node: SourceNode | None = None
        self._default_template_name: str | None = None

    def load_config(self, args: Namespace) -> None:
        """
        Loads the configuration for dragonglass.

        Args:
            args (Namespace): The command line arguments to dragonglass.
        """
        config_filename: str = str(args.config) if args.config else DEFAULT_CONFIG_FILE
        assert self.source_dir is not None
        config_path = self.source_dir / config_filename
        if config_path.exists() and config_path.is_file():
            with open(config_path, "rb") as f:
                self.config = tomllib.load(f)
        templates_section = self.config.get("templates", {})
        self.template_dir = self.source_dir / templates_section.get("directory", DEFAULT_TEMPLATE_DIRECTORY)
        self._default_template_name = templates_section.get("default", DEFAULT_TEMPLATE_NAME)

    @property
    def url_prefix(self) -> str:
        """Returns the configured URL prefix for all URLs generated for internal links."""
        links_section = self.config.get("links", {})
        rc = links_section.get("prefix", "/")
        return rc if rc.endswith("/") else rc + '/'

    @property
    def relative_links(self) -> bool:
        """
        Returns ``True`` if we generate relative links for internal links, ``False`` if we generate absolute links.
        """
        links_section = self.config.get("links", {})
        return links_section.get("relative", False)

    @property
    def default_stylesheet(self) -> str:
        """Returns the default stylesheet name."""
        templates_section = self.config.get("templates", {})
        return templates_section.get("stylesheet", DEFAULT_STYLESHEET_NAME)

    def get_template_name_for_node(self, node: SourceNode) -> str:
        """
        Returns the name of a template to be used to render the current node.

        Args:
            node (SourceNode): The current node being processed.

        Returns:
            str: TRhe name of the template to be used to render the node.
        """
        if not self.template_dir.is_dir():
            return DEFAULT_TEMPLATE_NAME
        return node.metadata.get("template", self._default_template_name)
